%% Example 3: Verification Against Analytical Solution
% In this example we consider the analytic solution to the Poisson equation
% in an infinite domain constructed as the imaginary part of an analytic
% function f(z) = z + .5z^2, i.e., p(x,y) = Img(f(x+iy)). By prescribing
% this solution along the boundary of a domain, we obtain a Dirichlet
% problem with a known solution, which we solve approximately on two
% different grids (a triangular and a Voronoi grid) to verify the mimetic,
% the two-point, and the mpfa solvers in MRST.

mrstModule add mpfa;

%%
% Construct repeatable sequence of pseudo random numbers.  If you want to
% reproduce the exact results from the test, then you should uncomment the
% following two statements.  Do note that the <matlab:doc('RandStream')
% |RandStream|> class was not introduced until MATLAB 7.7 (R2008b).  If you
% are using an older edition of MATLAB, you can produce repeatable random
% numbers by means of the |'state'| or |'seed'| options to the
% <matlab:doc('rand') |rand|> function.
%
% You may also want to uncomment the statement at the end of this script to
% restore the original random number generator state upon completion of the
% script.

% MT19937AR  = RandStream('mt19937ar', 'seed', 10);
% DfltStream = RandStream.setDefaultStream(MT19937AR);

%% Make generating point set
% We use the point-set 'seamount.mat' supplied with Matlab to make a
% triangular grid and a Voronoi grid. The point-set is centred around the
% origin and then perturbed slightly to avoid the occurence of certain
% degeneracies in the Voronoi grid.
a   = load('seamount');
a.x = a.x - mean(a.x(:)) + 1e-5*rand(numel(a.x),1);
a.y = a.y - mean(a.y(:)) + 1e-5*rand(numel(a.y),1);

%% Define analytical solution in the infinite domain
zsol =@(z) z+0.5*z.^2;
psol =@(x) imag(zsol(complex(x(:,1),x(:,2))));

%% Loop over triangular and Voronoi grid
for i=1:2
   
   %% Make grid
   if i==1
      G = triangleGrid([a.x(:) a.y(:)]);
   else
      G = pebi(triangleGrid([a.x(:) a.y(:)]));
   end
   G = computeGeometry(G);
   
   %% Set boundary conditions
   % The pressure solvers in MRST assume that all pressures are analytic.
   % Hence we translate the Dirichlet data so that they become nonnegative.
   bfxy  = find(any(G.faces.neighbors == 0,2));
   pbc   = psol(G.faces.centroids(bfxy,:));
   bcmin = min(pbc);
   bc    = addBC([],bfxy,'pressure', pbc - bcmin, 'sat', 1);

   %% Rock and fluid data
   fluid  = initSingleFluid('mu', 1, 'rho', 0);
   rock.perm = ones(G.cells.num,1);

   %% Compute and visualize approximate solutions
   % We compute the approximate solution to a problem with positive
   % boundary values, and translate the solution back again.
   figure,
   subplot(2,2,1), title('analytical');
   pa = psol(G.cells.centroids);
   plotCellData(G, pa); axis tight off; colorbar

   subplot(2,2,2), title('error: TPFA');
   T = computeTrans(G, rock);
   xi = initResSol(G, 350*barsa, 0.0);
   xtp = incompTPFA(xi, G, T, fluid, 'bc', bc);
   plotCellData(G, xtp.pressure + bcmin - pa); axis tight off; colorbar

   subplot(2,2,3), title('error: mimetic');
   S = computeMimeticIP(G, rock, 'InnerProduct', 'ip_simple');
   xmim = solveIncompFlow(xi, G, S, fluid, 'bc', bc);
   plotCellData(G, xmim.pressure + bcmin - pa); axis tight off; colorbar

   subplot(2,2,4), title('error: MPFA');
   Tm = computeMultiPointTrans(G, rock);
   xmp = incompMPFA(xi, G, Tm, fluid, 'bc', bc);
   plotCellData(G, xmp.pressure + bcmin - pa); axis tight off; colorbar
   
   %% Compute and report errors
   if i==1
      disp('L1 error on triangular grid');
   else
      disp('L1 error on Voronoi grid'); 
   end
   fprintf(1, '\tTPFA   :   %.3e\n', ...
      sum(abs(xtp.pressure + bcmin - pa).*G.cells.volumes)./...
      sum(G.cells.volumes));
   fprintf(1, '\tMimetic:   %.3e\n',...
      sum(abs(xmim.pressure + bcmin - pa).*G.cells.volumes)./...
      sum(G.cells.volumes));
   fprintf(1, ' \tMPFA   :   %.3e\n\n', ...
      sum(abs(xmp.pressure + bcmin - pa).*G.cells.volumes)./...
      sum(G.cells.volumes));
end

%%
% Restore initial state of random number generator.

% RandStream.setDefaultStream(DfltStream);
