%% Example 4: Realistic Model
% In this script we consider a single layer of a real-field model that has
% been flattened and modified so that all pillars are vertical and the
% layer has constant thickness. Wells are modelled as point sources and in
% an infinite domain, we then known the exact analytical pressure solution.
% By imposing this solution along the outer boundary, we obtain an
% analytical solution on a grid with a large degree of realism.

%% Load data and set parameters
% The original data model is confidential and therefore we can only provide
% the grid structure representing the modified layer (and not the data used
% to generate it).
dd = fileparts(mfilename('fullpath'));

load(fullfile(dd, 'data', 'realModel-L01'));
addpath(fullfile(dd, 'utils'));               clear dd

pmin = 100*barsa();
fluid = initSingleFluid('mu', 1*centi*poise, 'rho', 1000*kilogram/meter^3);

%% Find lateral boundary
bfx = G.cells.faces(G.cells.faces(:,2)<5, 1);
bf = bfx(any(G.faces.neighbors(bfx,:) == 0, 2));
cbfxy = find(G.cells.faces(:,2) < 5);
ci = find(any(G.faces.neighbors(G.cells.faces(cbfxy,1),:) == 0,2));
bfxy = G.cells.faces(cbfxy(ci),1);

%% Set analytical solution at outer boundaries
max_pos = max(G.faces.centroids);
min_pos = min(G.faces.centroids);
mu=fluid.properties();
lamperm = rock.perm(1,1)/mu;

psol =@(x) analytic2dSolutionWell(W, G, x, lamperm);
psol =@(x) psol(x)-min(psol(G.faces.centroids(bfxy,:)))+pmin;
bc = addBC([],bfxy,'pressure',psol(G.faces.centroids(bfxy,:)),'sat',1);


%% Compute solutions
% Init state structure 
r0 = initResSol(G, 350*barsa, 0.0);

% TPFA solution
T=computeTrans(G,rock);
rt = incompTPFA(r0, G, T, fluid, 'wells', W, 'bc', bc);

% Mimetic solution
S = computeMimeticIP(G,rock,'InnerProduct','ip_quasirt');
rm = solveIncompFlow(r0, G, S, fluid,'wells',W,'bc',bc);

% Analytic solution
refp = psol(G.cells.centroids);

%% Plot results
clf
for i=1:numel(W), W(i).name=num2str(i); end                            %#ok
nw_cells = true(G.cells.num,1);
nw_cells([W(:).cells]) = false;
pmax = max([max(refp(nw_cells)) max(rt.pressure) max(rm.pressure)]);
p = get(gcf,'position'); p(3:4) = [840 280]; set(gcf,'position', p);

subplot('position',[.025 0 .3 1])
plotCellData(G, convertTo(refp, barsa))
plotWell(G,W, 'color','k');
axis equal tight off, caxis(convertTo([pmin pmax], barsa))
text(4.565e5, 7.326e6, 'Analytic')

subplot('position',[.350 0 .3 1])
plotCellData(G, convertTo(rt.pressure, barsa))
plotWell(G,W,'color','k');
axis equal tight off, caxis(convertTo([pmin pmax], barsa))
text(4.565e5, 7.326e6, 'TPFA')

subplot('position',[.675 0 .3 1])
plotCellData(G, convertTo(rm.pressure, barsa))
plotWell(G,W,'color','k');
axis equal tight off, caxis(convertTo([pmin pmax], barsa))
text(4.565e5, 7.326e6, 'Mimetic')

h = colorbar('vert'); set(h,'position', [.93 .05 .015 .35]);
% colormap(jet(16));
colormap((flipud(hot(32))+2*jet(32))/3), brighten(-.5)

%% Plot errors
clf, colormap(jet(128))
errt = convertTo(rt.pressure-refp, barsa);
errm = convertTo(rm.pressure-refp, barsa);

subplot(1,2,1),
plotCellData(G,errt, nw_cells), axis tight, title('TPFA'),
plotWell(G,W,'color','k'); colorbar

subplot(1,2,2),
plotCellData(G,errm, nw_cells), axis tight, title('Mimetic')
plotWell(G,W,'color','k'); colorbar


%% Make tabular of realtive error for discretizations
% The error is measured as the discrepancy between the analytical solution
% and the numerical solution in all cells except for the well cells (where
% the analytical solution is infinite). The discrepancy is then scaled by
% the span in values for the analytical solution
disp(' '); disp('Error table'); disp('-------------------------');
p     = refp(~isinf(refp)); 
scale = max(p) - min(p); clear p
errt = abs(errt(~isinf(errt)));
errm = abs(errm(~isinf(errm)));
fprintf(1, 'TPFA:\t%f\n', max(errt) / convertTo(scale, barsa));
fprintf(1, 'IP_QRT:\t%f\n', max(errm) / convertTo(scale, barsa));

mrstModule add mpfa
Tm  = computeMultiPointTrans(G,rock);
r   = incompMPFA(r0, G, Tm, fluid,'wells',W,'bc',bc);
err = r.pressure - refp;
err = abs(err(~isinf(err)));
fprintf(1, 'MPFA-O:\t%f\n', max(err) / scale);

fprintf(1, 'IP_QFAMILY(t):\n');
% Compute solution for for mimetic inner product family and report errors
for param=[0.5 1:7 10 100]
   S   = computeMimeticIP(G, rock, 'InnerProduct', 'ip_dimfamily', ...
                          'Inner_Param',param);
   r   = solveIncompFlow(r0, G, S, fluid,'wells',W,'bc',bc);
   err = r.pressure - refp;
   err = abs(err(~isinf(err)));
   fprintf(1, '%.1f:\t%f\n', param, max(err) / scale);
end

