%% Figure 4: Putting it all together
% This script is the first example of a MRST solver in which we go through
% the steps necessary to solve a single-phase pressure equation. The
% example is discussed in Section 2.4

%% Setup parameters
nx = 20; ny = 20; nz = 10;
G = computeGeometry(cartGrid([nx, ny, nz]));
rock.perm = repmat(100 * milli*darcy, [G.cells.num, 1]);
fluid = initSingleFluid('mu', 1*centi*poise, 'rho', 1014*kilogram/meter^3);
gravity reset on

%% Fluid sources
% We place a single column of fluid sources in the middle of the reservoir
c   = (nx/2*ny+nx/2 : nx*ny : nx*ny*nz) .';
src = addSource([], c, ones(size(c)) ./ day());

%% Boundary conditions
% The solvers in MRST assume that all boundaries to have no-flow conditions
% unless specified otherwise. We therefore only need to set one condition
% at the outflow boundary (left boundary)
bc = pside([], G, 'LEFT', 10*barsa());

%% Compute mimetic inner products
S = computeMimeticIP(G, rock, 'Verbose', true);

%% Initialize and solve problem
rSol = initResSol(G, 0);
rSol = solveIncompFlow(rSol, G, S, fluid,'src', src, 'bc', bc);
p = convertTo(rSol.pressure(1:G.cells.num), barsa() );

%% Plot solution
clf,
pos = get(gcf,'position'); pos(3:4) = [840 420]; set(gcf,'position',pos);

% Problem setup
subplot(1,2,1)
plotGrid(G,'FaceColor','none');
plotFaces(G,bc.face,'FaceColor','b');
plotGrid(G,c,'FaceColor','r');
view(-40,40), axis equal tight off

% Solution
subplot(1,2,2)
plotCellData(G,p);
view(-40,40), axis equal tight
colorbar('horiz');
