function figure9()
% Make the contents of Figure 9 illustrating lack of monotonicity for three
% different mimetic inner products: IP_SIMPLE, IP_QRT and IP_QTPF.
% Computational setup as in Figure 8.

   MODS = mrstModule;
   mrstModule add streamlines
   addpath(fullfile(fileparts(mfilename('fullpath')), 'utils'))

   %% Make grid
   G = twister(cartGrid([11,11]), 0.03);
   G = computeGeometry(G);
   
   %% Make anisotropic permeability tensor
   t     =  30*pi/180;
   U     = [ cos(t), sin(t);
            -sin(t), cos(t)];
   Kd    = diag([1,1000]);
   K     =  U'*Kd*U;
   rocktensor =[K(1,1),K(1,2),K(2,2)]*milli*darcy();
   rock.perm = repmat(rocktensor, [G.cells.num, 1]);
   rock.poro = ones(G.cells.num, 1);

   %% Set fluid properties and boundary conditions
   fluid     = initSingleFluid('mu' ,    1*centi*poise     , ...
                               'rho', 1014*kilogram/meter^3);
   gravity off
   bc = [];
   bc = pside(bc, G, 'LEFT',  100*barsa);
   bc = pside(bc, G, 'RIGHT', 10*barsa);
   bc.sat = 1*bc.value;
   
   
   %% APPROACH 1: Mimetic with SIMPLE-inner product
   x1 = initResSol(G, 0.0);
   IP = computeMimeticIP(G, rock, 'Verbose', true, ...
      'InnerProduct', 'ip_simple');
   x1 = solveIncompFlow(x1, G, IP, fluid, 'bc', bc);

   %% APPROACH 2: Mimetic with QUASIRT-inner product
   x2 = initResSol(G, 0.0);
   IP = computeMimeticIP(G, rock, 'Verbose', true, ...
      'InnerProduct', 'ip_quasirt');
   x2 = solveIncompFlow(x2, G, IP, fluid, 'bc', bc);
   
   %% APPROACH 3: Mimetic with QUASITPF-inner product
   x3 = initResSol(G, 0.0);
   IP = computeMimeticIP(G, rock, 'Verbose', true, ...
      'InnerProduct', 'ip_quasitpf');
   x3 = solveIncompFlow(x3, G, IP, fluid, 'bc', bc);
   

   %% Streamline plots
   clf;
   c = (ceil(G.cartDims(1)/2):G.cartDims(1):prod(G.cartDims))';
   opt = {'maxsteps', 1000};
   
   subplot(1,3,1)
   r=x1;           streamline(pollock(G, r, c, opt{:}));
   r.flux=-r.flux; streamline(pollock(G, r, c, opt{:}));
   plotOutline(G);
   axis equal, a=axis; a=a+[-1,1,-1,1]*0.5; axis(a), axis tight
   title('SIMPLE'), xlabel('x')
   
   subplot(1,3,2)
   r=x2;           streamline(pollock(G, r, c,opt{:}));
   r.flux=-r.flux; streamline(pollock(G, r, c, opt{:}));
   plotOutline(G);
   axis equal, a=axis; a=a+[-1,1,-1,1]*0.5; axis(a), axis tight
   title('QRT'), xlabel('x')
   
   subplot(1,3,3)
   r=x3;           streamline(pollock(G, r, c,opt{:}));
   r.flux=-r.flux; streamline(pollock(G, r, c, opt{:}));
   plotOutline(G);
   axis equal, a=axis; a=a+[-1,1,-1,1]*0.5; axis(a), axis tight
   title('QTPFA'), xlabel('x')
   
   set(gcf, 'renderer', 'painters');

   mrstModule clear
   mrstModule('add', MODS{:})
   rmpath(fullfile(fileparts(mfilename('fullpath')), 'utils'))
end
