%% Table 2: Compute well constants for various inner products
% The equivalent radius used in the Peaceman well model is defined as
%
%    $r_0(IP; \beta) = C(IP; \beta)(x^2 + y^2)^{1/2}$
%
% Where the constant C(IP; $\beta$) depends on the numerical method used
% (in this case the inner product) and the grid aspect ratio
%
%     $\beta = \delta x/\delta y$.
%
% In this script we compute C(IP; $\beta$) for the mimetic inner product
% familiy IP_QFAMILY for various aspect ratios.
%
% This is done by constructing and solving radial flow system. Then we
% compute r_0 from an analytical relation for the pressure drop, and
% compute C from the relation presented above.
%
% The simulation grid is cartesian and has dimension [2*n+1 2*m+1]
% giving $\beta = n/m$.

%% Set up parameters
cases     = [1 2 4 8 16 32 64]; % grid aspect ratios (beta):
innerProd = 'ip_dimfamily';   % mimetic inner product
family    = 1:6;              % free parameter in IP_QFAMILY inner product.
fluid     = initSingleFluid('mu' ,    1*centi*poise     , ...
                            'rho', 1014*kilogram/meter^3);   

C = zeros(numel(family), numel(cases));

%% Loop over grid aspect ratios
clc;
fprintf(1, 'Computing well constants:');
for i = 1:numel(cases)    
   %% Construct grid and define permeability 
   % NB: The values in the paper are computed with dim =  50, but to speed
   % ut the calculation we use dim = 5 here.
   dim    =  5;  
   beta   = cases(i);
   n      = dim;        m      = beta*dim; 
   deltaX = 1/(2*n+1);  deltaY = 1/(2*m+1);
   source = (2*n+1)*(m)+n+1;  % put source in the middle of the grid   
   
   G = cartGrid([2*n+1 2*m+1], [1 1]);
   G = computeGeometry(G);   
   rock.perm = ones(G.cells.num, 1)*milli*darcy;    
     
   %% Prescribe boundary conditions
   % find bc faces
   westFaces  = find(G.faces.centroids(:,1) == 0);
   eastFaces  = find(G.faces.centroids(:,1) == 1);
   southFaces = find(G.faces.centroids(:,2) == 0);
   northFaces = find(G.faces.centroids(:,2) == 1); 
   boundaryFaces = [westFaces; eastFaces; southFaces; northFaces];
   
   % Force radial flow by prescribing flux bc values according to angle
   % theta between the corners on the bondary face,
   % Fi = q*theta_i=/2*pi
   thetas = zeros(length(boundaryFaces), 1);
   center = G.cells.centroids(source,1:2);
   for k = 1:length(boundaryFaces)
      face = boundaryFaces(k);
      nodes  = G.faces.nodes(G.faces.nodePos(face):(G.faces.nodePos(face+1)-1));
      coords = G.nodes.coords(nodes, 1:2);      
     
      points = unique(coords, 'rows');  % the corners on the boundary faces
      v1 = points(1,:) - center;    % distance from corners of bf to source
      v2 = points(2,:) - center;
      theta = abs(acos( (v1*v2')/(norm(v1)*norm(v2)) ));
      thetas(k) = theta;
   end
   bc = addBC([], boundaryFaces, 'flux', -thetas/(2*pi));   
   
   %% prescribe source
   q = sum(thetas/(2*pi));
   src = addSource([], source, q);   
   
   
   cell   = source+n;   % choose boundary cell to compute pressure drop
   r_face = (n)*deltaX; % radius from source to centroid of boundary cell
     
   %% Loop over familiy of inner products
   for j = family
      S = computeMimeticIP(G, rock, 'Verbose',false,  'InnerProduct', ...
                           innerProd, 'Inner_Param', j);
      
      % Initialize and solve system
      x = initResSol(G, 0.0);    
      x = solveIncompFlow(x, G, S, fluid, 'src', src, 'bc', bc);     
      
      % pressure drop
      delP = x.pressure(cell)-x.pressure(source);
      
      %% Compute r_0 and C(ip, beta)
      mu  = fluid.properties(x);
      r0  = r_face*exp(2*pi*delP*rock.perm(1)/(q*mu));
      C(j, i) = r0/sqrt(deltaX^2+deltaY^2);   
      fprintf(1, '.');
   end   
end

%% Print table
fprintf(1, '\n\nbeta');
fprintf(1, '%7.0f ', cases);
fprintf(1, '\n');
for k = family
fprintf(1, 't = %.0f \t', k);  
fprintf(1, '%7.4f ', C(k, :));
fprintf(1, '\n');
end
