function monotonicity_streamlines(G, K, c)
% Simulate single-phase flow in a square domain given by the grid G for a
% homogeneous, anisotropic permeability K. The flow is driven by a pressure
% drop from left to right (Dirichlet boundary conditions pL = 100 bar and
% pR = 10 bar) and no-flow boundary conditions along the top and bottom.
% Solutions are computed for the TPFA method, the mimetic method, and the
% MPFA-O method and visualized by tracing streamlines backward and forward
% from the positions in c and by plotting pressure values at cell centers.

   MODS = mrstModule;
   mrstModule add mpfa streamlines
  
   %% Compute grid geometry and set up rock structure
   G = computeGeometry(G);

   rock.perm = repmat([K(1,1),K(1,2),K(2,2)]*milli*darcy(), [G.cells.num, 1]);
   rock.poro = ones(G.cells.num, 1);

   %% Set fluid properties and boundary conditions
   fluid     = initSingleFluid('mu' ,    1*centi*poise     , ...
                               'rho', 1014*kilogram/meter^3);
   gravity off
   bc = [];
   bc = pside(bc, G, 'LEFT',  100*barsa);
   bc = pside(bc, G, 'RIGHT', 10*barsa);
   bc.sat = 1*bc.value;
   
   %% APPROACH 1: Two-point flux
   x1 = initResSol(G, 0.0);
   T  = computeTrans(G, rock, 'Verbose', true);
   x1 = incompTPFA(x1, G, T, fluid, 'Verbose', true, 'bc',bc);
   
   %% APPROACH 2: Mimetic with SIMPLE-inner product
   x2 = initResSol(G, 0.0);
   IP = computeMimeticIP(G, rock, 'Verbose', true, ...
      'InnerProduct', 'ip_simple','inner_param', 6);
   x2 = solveIncompFlow(x2, G, IP, fluid,'bc',bc);
   
   %% APPROACH 3: MPFA method
   x3 = initResSol(G, 0.0);
   T_mpfa = computeMultiPointTrans(G, rock, 'Verbose', true);
   x3 = incompMPFA(x3, G, T_mpfa, fluid,'bc',bc);
   
   
   %% Streamline plots
   clf,
   opt = {'maxsteps', 1000};
   
   subplot(2,3,1)
   r=x1;           streamline(pollock(G, r, c, opt{:}));
   r.flux=-r.flux; streamline(pollock(G, r, c, opt{:}));
   plotOutline(G);
   axis equal, a=axis; a=a+[-1,1,-1,1]*0.5; axis(a), axis tight
   title('TPFA')
   
   subplot(2,3,2)
   r=x2;           streamline(pollock(G, r, c,opt{:}));
   r.flux=-r.flux; streamline(pollock(G, r, c, opt{:}));
   plotOutline(G);
   axis equal, a=axis; a=a+[-1,1,-1,1]*0.5; axis(a), axis tight
   title('mimetic');
   
   subplot(2,3,3)
   r=x3;           streamline(pollock(G, r, c,opt{:}));
   r.flux=-r.flux; streamline(pollock(G, r, c, opt{:}));
   plotOutline(G);
   axis equal, a=axis; a=a+[-1,1,-1,1]*0.5; axis(a), axis tight
   title('MPFA');
   
   %% Pressure plots
   subplot(2,3,4)
   plotCellData(G, convertTo(x1.pressure, barsa), 'EdgeAlpha',.1);
   a=axis; a=a+[-1,1,-1,1]; axis(a), axis tight, xlabel('x')
   p=get(gca,'pos'); p(2)=p(2)+.05; set(gca,'position',p);
   
   subplot(2,3,5)
   plotCellData(G, convertTo(x2.pressure, barsa), 'EdgeAlpha',.1);
   a=axis; a=a+[-1,1,-1,1]; axis(a), axis tight, xlabel('x')
   p=get(gca,'pos'); p(2)=p(2)+.05; set(gca,'position',p);
     
   subplot(2,3,6)
   plotCellData(G, convertTo(x3.pressure, barsa), 'EdgeAlpha',.1);
   a=axis; a=a+[-1,1,-1,1]; axis(a), axis tight, xlabel('x')
   p=get(gca,'pos'); p(2)=p(2)+.05; set(gca,'position',p);
   h = colorbar('vert'); set(h,'position', [.93 .1595 .015 .34285]);
   
   set(gcf, 'renderer', 'painters');
   mrstModule clear
   mrstModule('add', MODS{:})   
end
